{% extends "touchtechnology/admin/edit.html" %}
{% load common %}
{% load cloudinary %}

{% block form-fieldset %}
	<fieldset>
		{% cloudinary object.image width="800" format="jpg" sign_url=True %}
		{% for f in form %}
			{% field f %}
		{% endfor %}
	</fieldset>
{% endblock form-fieldset %}
