# -*- coding: utf-8 -*-
# Generated by Django 1.11.5 on 2017-09-25 08:13
from __future__ import unicode_literals

import django.contrib.postgres.fields.jsonb
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("common", "0003_drop_pythonpackage"),
    ]

    operations = [
        migrations.AddField(
            model_name="sitemapnode",
            name="kwargs",
            field=django.contrib.postgres.fields.jsonb.JSONField(default=dict),
        ),
    ]
